
# coding: utf-8

# In[1]:


import matplotlib.pyplot as plt
import numpy as np
plt.close('all')





x= np.loadtxt('signal.txt')
N=x.size

plt.figure()
plt.plot(x)
plt.show()


# On calcule la transformée en ondelettes du signal 

# In[4]:


# on charge la librairie ondelettes
import pywt

ond='haar'
# on calcule l'ondelette de base
wavelet = pywt.Wavelet(ond)

#on calcule les coefficients de decomposition dans la base d'ondelette
Ca,Cd = pywt.dwt(x, wavelet, mode = "periodization")


# On trace les coefficients en ondelettes calculés avec l'ondelette de Haar 

# In[5]:


coeffs=np.concatenate((Ca,Cd))
plt.figure()
plt.plot(coeffs)
plt.show()


# On réitère l'opération sur les coefficients d'approximation.

# In[6]:


Caa,Cad=pywt.dwt(Ca, wavelet, mode = "periodization")

coeffs=np.concatenate((Caa,Cad,Cd))
plt.figure()
plt.plot(coeffs)
plt.show()


# Les commandes suivantes font cette opération directement.

# In[7]:


Caa,Cad,Cd = pywt.wavedec(x, ond, level=2)

coeffs=np.concatenate((Caa,Cad,Cd))
plt.figure()
plt.plot(coeffs)
plt.show()


# On effectue une decomposition sur 7 niveaux pour ensuite chercher à obtenir une approximation parcimonieuse. L'approximation est ici de taille $512/2^7=2^9/2^7=2^2$.

# In[8]:


n0=7
coeffs_l=pywt.wavedec(x, ond, level=n0)
nsig=np.floor(np.log2(N))

#calcul de la taille de l'approximation
na=np.floor(nsig-n0)
na=np.int(na)
print(na)

#le resultat est une liste. On concatene pour obtenir un vecteur numpy
coeffs=np.concatenate(coeffs_l)

plt.figure()
plt.plot(coeffs)
plt.show()


# On calcule maintenant une approximation avec les $M$ premiers vecteurs de base.

# In[9]:


M=2**na

coeffs_M=coeffs_l.copy()
arr,sli=pywt.coeffs_to_array(coeffs_M)

#On garde seulement les M premiers coefficients et met à zéro tous les autres
arr_M=np.zeros(arr.size)
arr_M[0:M]=arr[0:M].copy()

#on reconstruit l'approximation du signal à partir de ses coefficients
coeffs_M=pywt.array_to_coeffs(arr_M, sli,output_format='wavedec')

x_M=pywt.waverec(coeffs_M, ond)

plt.figure()
plt.plot(x_M)
plt.show()


# In[10]:


M=2**6

coeffs_M=coeffs_l.copy()
arr,sli=pywt.coeffs_to_array(coeffs_M)

#On garde seulement les M premiers coefficients et met à zéro tous les autres
arr_M=np.zeros(arr.size)
arr_M[0:M]=arr[0:M].copy()

#on reconstruit l'approximation du signal à partir de ses coefficients
coeffs_M=pywt.array_to_coeffs(arr_M, sli,output_format='wavedec')

x_M=pywt.waverec(coeffs_M, ond)

plt.figure()
plt.plot(x_M)
plt.show()


# On construit maintenant une approximation non linéaire: on garde les coefficients de détail au dessus d'un certain seuil.

# In[11]:


arr_T=arr.copy()

#On met en mémoire l'approximation qu'on ne touchera pas.
arr_Ta=arr[0:na].copy()

#On met a zero l'approximation
arr_T[0:na]=np.zeros(na)

arr_Ts=np.sort(np.abs(arr_T))
arg_Ts=np.argsort(np.abs(arr_T))

plt.figure(8)
plt.plot(arr_Ts)
plt.show()


# On garde les $M$ coefficients les plus gros

# In[12]:


M=2**6

arr_T[arg_Ts[0:N-M]]=np.zeros(N-M)

arr_T[0:na]=arr_Ta

plt.figure(9)
plt.plot(arr)
plt.plot(arr_T)
plt.show()

#on reconstruit l'approximation du signal à partir de ses coefficients
coeffs_nT=pywt.array_to_coeffs(arr_T, sli,output_format='wavedec')

x_nT=pywt.waverec(coeffs_nT, ond)

plt.figure(10)
plt.plot(x_nT)
plt.show()


# In[13]:


M=2**7

arr_T=arr.copy()

#On met en mémoire l'approximation qu'on ne touchera pas.
arr_Ta=arr[0:na].copy()

#On met a zero l'approximation
arr_T[0:na]=np.zeros(na)

#On classe les coefficients de facon croissante
arr_Ts=np.sort(np.abs(arr_T))
arg_Ts=np.argsort(np.abs(arr_T))

plt.figure(11)
plt.plot(arr_Ts)
plt.show()

# On met a zero les N-M plus petits coefficients de details et on garde les autres
arr_T[arg_Ts[0:N-M]]=np.zeros(N-M)

arr_T[0:na]=arr_Ta

plt.figure(12)
plt.plot(arr)
plt.plot(arr_T)
plt.show()

#on reconstruit l'approximation du signal à partir de ses coefficients
coeffs_nT=pywt.array_to_coeffs(arr_T, sli,output_format='wavedec')

x_nT=pywt.waverec(coeffs_nT, ond)

plt.figure(13)
plt.plot(x_nT)
plt.show()

