
# coding: utf-8

# On charge un fichier .txt a l'aide de numpy

# In[1]:


import matplotlib.pyplot as plt
import numpy as np

plt.close('all')

# In[2]:


signalf= np.loadtxt('signal.txt')
N=signalf.size

plt.figure()
plt.plot(signalf)
plt.show()


# On calcule la transformée en ondelettes du signal à l'aide de la librairie pywt

# In[3]:


# on charge la librairie ondelettes
import pywt

# on calcule l'ondelette de base
wavelet = pywt.Wavelet('haar')

#on calcule les coefficients de decomposition dans la base d'ondelette
Ca,Cd = pywt.dwt(signalf, wavelet, mode = "periodization")


# On trace les coefficients en ondelettes calculés avec l'ondelette de Haar 

# In[4]:


coeffs=np.concatenate((Ca,Cd))
plt.figure()
plt.plot(coeffs)
plt.show()


# On réitère l'opération sur les coefficients d'approximation.

# In[5]:


Caa,Cad=pywt.dwt(Ca, wavelet, mode = "periodization")

coeffs=np.concatenate((Caa,Cad,Cd))
plt.figure()
plt.plot(coeffs)
plt.show()


# Les commandes suivantes font cette opération directement.

# In[6]:


Caa,Cad,Cd = pywt.wavedec(signalf, 'haar', level=2)

coeffs=np.concatenate((Caa,Cad,Cd))
plt.figure()
plt.plot(coeffs)
plt.show()


# On effectue une decomposition sur 7 niveaux pour ensuite chercher à obtenir une approximation parcimonieuse. L'approximation est ici de taille $512/2^7=2^9/2^7=2^2$.

# In[7]:


n0=7
coeffs_l=pywt.wavedec(signalf, 'haar', level=n0)
nsig=np.floor(np.log2(N))

#calcul de la taille de l'approximation
na=np.floor(nsig/2**n0)
na=np.int(na)

#le resultat est une liste. On concatene pour obtenir un vecteur numpy
coeffs=np.concatenate(coeffs_l)

plt.figure()
plt.plot(coeffs)
plt.show()


# On calcule maintenant une approximation avec les $M$ premiers vecteurs de base.

# In[8]:


M=2**2

coeffs_M=coeffs_l.copy()
arr,sli=pywt.coeffs_to_array(coeffs_M)

#On garde seulement les M premiers coefficients et met à zéro tous les autres
arr_M=np.zeros(arr.size)
arr_M[0:M]=arr[0:M].copy()

#on reconstruit l'approximation du signal à partir de ses coefficients
coeffs_M=pywt.array_to_coeffs(arr_M, sli)

signal_M=pywt.waverecn(coeffs_M, 'haar')

plt.figure()
plt.plot(signal_M)
plt.show()


# In[9]:


M=2**6

coeffs_M=coeffs_l.copy()
arr,sli=pywt.coeffs_to_array(coeffs_M)

#On garde seulement les M premiers coefficients et met à zéro tous les autres
arr_M=np.zeros(arr.size)
arr_M[0:M]=arr[0:M].copy()

#on reconstruit l'approximation du signal à partir de ses coefficients
coeffs_M=pywt.array_to_coeffs(arr_M, sli)

signal_M=pywt.waverecn(coeffs_M, 'haar')

plt.figure()
plt.plot(signal_M)
plt.show()


# On construit maintenant une approximation non linéaire: on garde les coefficients de détail au dessus d'un certain seuil.

# In[10]:


arr_T=arr.copy()

#On met en mémoire l'approximation qu'on ne touchera pas.
arr_Ta=arr[0:na].copy()

#On met a zero l'approximation
arr_T[0:na]=np.zeros(na)

arr_Ts=np.sort(np.abs(arr_T))
arg_Ts=np.argsort(np.abs(arr_T))

plt.figure()
plt.plot(arr_Ts)
plt.show()


# On garde les $M$ coefficients les plus gros

# In[11]:


M=2**6

arr_T[arg_Ts[0:N-M]]=np.zeros(N-M)

arr_T[0:na]=arr_Ta

plt.figure()
plt.plot(arr)
plt.plot(arr_T)
plt.show()

#on reconstruit l'approximation du signal à partir de ses coefficients
coeffs_nT=pywt.array_to_coeffs(arr_T, sli)

signal_nT=pywt.waverecn(coeffs_nT, 'haar')

plt.figure()
plt.plot(signal_nT)
plt.show()


# In[14]:


M=2**7

arr_T=arr.copy()

#On met en mémoire l'approximation qu'on ne touchera pas.
arr_Ta=arr[0:na].copy()

#On met a zero l'approximation
arr_T[0:na]=np.zeros(na)

arr_Ts=np.sort(np.abs(arr_T))
arg_Ts=np.argsort(np.abs(arr_T))

plt.figure()
plt.plot(arr_Ts)
plt.show()

arr_T[arg_Ts[0:N-M]]=np.zeros(N-M)

arr_T[0:na]=arr_Ta

plt.figure()
plt.plot(arr)
plt.plot(arr_T)
plt.show()

#on reconstruit l'approximation du signal à partir de ses coefficients
coeffs_nT=pywt.array_to_coeffs(arr_T, sli)

signal_nT=pywt.waverecn(coeffs_nT, 'haar')

plt.figure()
plt.plot(signal_nT)
plt.show()

